//+------------------------------------------------------------------+
//|                                              Moving Averages.mq5 |
//|              Copyright Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#include "ExpertAdvisor.mqh"

input double Risk      = 0.1; // 
input int    SL        = 100; //  
input int    TP        = 100; //  
input int    TS        =  30; //   
input int    pMA       =  12; //   
input int    HourStart =   7; //   
input int    HourEnd   =  20; //   
//---
class CMyEA : public CExpertAdvisor
  {
protected:
   double            m_risk;          //  
   int               m_sl;            // 
   int               m_tp;            // 
   int               m_ts;            // 
   int               m_pMA;           // MA 
   int               m_hourStart;     //   
   int               m_hourEnd;       //   
   int               m_hma;           //  
public:
   void              CMyEA();
   void             ~CMyEA();
   virtual bool      Init(string smb,ENUM_TIMEFRAMES tf); // 
   virtual bool      Main();                              //  
   virtual void      OpenPosition(long dir);              //    
   virtual void      ClosePosition(long dir);             //    
   virtual long      CheckSignal(bool bEntry);            //  
  };
//------------------------------------------------------------------	CMyEA
void CMyEA::CMyEA() { }
//------------------------------------------------------------------	~CMyEA
void CMyEA::~CMyEA()
  {
   IndicatorRelease(m_hma); //   
  }
//------------------------------------------------------------------	Init
bool CMyEA::Init(string smb,ENUM_TIMEFRAMES tf)
  {
   if(!CExpertAdvisor::Init(0,smb,tf)) return(false);  //   

   m_risk=Risk; m_tp=TP; m_sl=SL; m_ts=TS; m_pMA=pMA;  //  
   m_hourStart=HourStart; m_hourEnd=HourEnd;

   m_hma=iMA(m_smb,m_tf,m_pMA,0,MODE_SMA,PRICE_CLOSE); //   
   if(m_hma==INVALID_HANDLE) return(false);            //    
   m_bInit=true; return(true);                         // " "
  }
//------------------------------------------------------------------	Main
bool CMyEA::Main() //  
  {
   if(!CExpertAdvisor::Main()) return(false); //    

   if(Bars(m_smb,m_tf)<=m_pMA) return(false); //   

   if(!CheckNewBar()) return(true);           //   

                                              //   
   long dir;
   dir=ORDER_TYPE_BUY;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);
   dir=ORDER_TYPE_SELL;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);

   return(true);
  }
//------------------------------------------------------------------	OpenPos
void CMyEA::OpenPosition(long dir)
  {
//   ,  
   if(PositionSelect(m_smb)) return;
   if(!CheckTime(StringToTime(IntegerToString(m_hourStart)+":00"),
      StringToTime(IntegerToString(m_hourEnd)+":00"))) return;
//      
   if(dir!=CheckSignal(true)) return;
   double lot=CountLotByRisk(m_sl,m_risk,0);
//    
   if(lot<=0) return;
//  
   DealOpen(dir,lot,m_sl,m_tp);
  }
//------------------------------------------------------------------	ClosePos
void CMyEA::ClosePosition(long dir)
  {
//   ,  
   if(!PositionSelect(m_smb)) return;
   if(!CheckTime(StringToTime(IntegerToString(m_hourStart)+":00"),
      StringToTime(IntegerToString(m_hourEnd)+":00")))
      //    ,   
     { m_trade.PositionClose(m_smb); return; }
//     
   if(dir!=PositionGetInteger(POSITION_TYPE)) return;
//        
   if(dir!=CheckSignal(false)) return;
//  
   m_trade.PositionClose(m_smb,1);
  }
//------------------------------------------------------------------	CheckSignal
long CMyEA::CheckSignal(bool bEntry)
  {
   MqlRates rt[2];
   if(CopyRates(m_smb,m_tf,0,2,rt)!=2)
     { Print("CopyRates ",m_smb,"   "); return(WRONG_VALUE); }

   double ma[1];
   if(CopyBuffer(m_hma,0,0,1,ma)!=1)
     { Print("CopyBuffer MA -  "); return(WRONG_VALUE); }

   if(rt[0].open<ma[0] && rt[0].close>ma[0])
      return(bEntry ? ORDER_TYPE_BUY:ORDER_TYPE_SELL); //   
   if(rt[0].open>ma[0] && rt[0].close<ma[0])
      return(bEntry ? ORDER_TYPE_SELL:ORDER_TYPE_BUY); //   

   return(WRONG_VALUE); //   
  }

CMyEA ea; //  
//------------------------------------------------------------------	OnInit
int OnInit()
  {
   ea.Init(Symbol(),Period()); //  

                               //  
// ea.Init(Symbol(), PERIOD_M5); //   
// ea.Init("USDJPY", PERIOD_H2); //     

   return(0);
  }
//------------------------------------------------------------------	OnDeinit
void OnDeinit(const int reason) { }
//------------------------------------------------------------------	OnTick
void OnTick()
  {
   ea.Main(); //   
  }
//+------------------------------------------------------------------+
